/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// HistogramDialog.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CHistogramDialog dialog

#define WM_HGGOODBYE WM_USER + 300
#define WM_HGUPDATE  WM_USER + 301

#include "GrabbedImageDialog.h"
#include "StaticFrame.h"
#include "TciGrabber.h"
#include "UHistogram.h"

class CHistogramDialog : public CDialog
{
	// Construction
private:
	CGrabbedImageDialog    *m_pGrabbedImgDlg;
	BOOL m_bGrabbedImgDlg;
	BOOL m_bFirstGrabbedImgDlg;
	VTciGrabber						*m_pGrabber;
	THistogram						 m_histo;
	TGaussianf						 m_colorModel;
public:
	int m_nThreshold;
	CStaticFrame	m_picProcessedImg;
	
private:
	int m_nSliderThreshold;
	int m_nArrayThreshold[256];
	CPoint m_pointGrabbedImgDlg;
	
	CDialog* m_pDlg;
	int Hide(CWnd* pWnd);
	int Show(CWnd* pWnd);
	
	int m_nFrameHeight;
	int m_nFrameWidth;
	BITMAPINFO_8 m_stdBmInfo;
public:
	CHistogramDialog(CWnd* pParent = NULL);   // standard constructor
	CHistogramDialog(CDialog* pDlg);
	~CHistogramDialog ();
	BOOL Create( int nPixSize, int nWidth, int nHeight, 
		VTciGrabber		*pGrabber,
		int nInitThreshold = 0,
		float fInitMaxDist = 0.0f );
	int		Hide();
	int		Show();
	THistogram*		Histo() { return &m_histo;}
	CRITICAL_SECTION*		Sem() { return &m_Sem;}
	int Threshold() { return m_nThreshold;}
	void	Quit() { OnClose();}
	
protected:
	long OnBMGoodbye(UINT wParam, long lParam);
	
	CRITICAL_SECTION		m_Sem;
	// Dialog Data
	//{{AFX_DATA(CHistogramDialog)
	enum { IDD = IDD_HISTOGRAM };
	float	m_fMaxDist;
	float	m_fResolution;
	int		m_nColorSource;
	BOOL	m_bRobust;
	//}}AFX_DATA
	
	
	// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CHistogramDialog)
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL
	
	// Implementation
protected:
	
	// Generated message map functions
	//{{AFX_MSG(CHistogramDialog)
	afx_msg void OnClose();
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	virtual BOOL OnInitDialog();
	afx_msg void OnColorsource();
	afx_msg void OnRadio7();
	afx_msg void OnGrab();
	afx_msg void OnCompute();
	afx_msg void OnSave();
	afx_msg void OnSetfromhisto();
	afx_msg void OnPaint();
	afx_msg void OnRobust();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
		
private:
	void CleanUp();
	
};
