///////////////////////////////////////////////////////////////////////////
// Filename	: ImgWndBB.cpp
// Created	: Tuesday, June 17th, 1997
// Description: A window class to display images with bounding box rectangle and eigen axes.
// 
//  Vasanth Philomin 1997, vasi@cs.umd.edu
///////////////////////////////////////////////////////////////////////////
// ImgWnd.cpp : implementation file
//

#include "stdafx.h"
#include "Grab.h"
#include "ImgWndBB.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CImgWndBB

CImgWndBB::CImgWndBB()
{
	m_bValidDib = NEVER;	// Dib is not valid at the beginning
	m_bDrawBB = FALSE;
	m_arrEigPoint.SetSize(4);	// Set size to 4 - Otherwise, there will be a dump if face
								// is not found the first time
}

CImgWndBB::~CImgWndBB()
{
	m_arrEigPoint.RemoveAll();
}


BEGIN_MESSAGE_MAP(CImgWndBB, CFrameWnd)
	//{{AFX_MSG_MAP(CImgWndBB)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CImgWndBB message handlers

BOOL CImgWndBB::Create(const char *strTitle, const CPoint &topLeftCorner, 
					   BITMAPINFO_8 &bmInfo, BOOL bMatchScreen, BOOL bDrawBB)
{
	DWORD dwStyle = WS_OVERLAPPEDWINDOW | WS_VISIBLE;
	// Create the dib
	m_dib.Create(bmInfo, bMatchScreen);
	CSize size = m_dib.GetDimensions();

	CRect rectWindow(topLeftCorner.x, topLeftCorner.y, topLeftCorner.x + size.cx,
							topLeftCorner.y + size.cy);

	// Inflate rectangle for the border
	rectWindow.InflateRect(0, 0, defaultBorderWidth, defaultBorderWidth);

	// Get change in size due to window style
	AdjustWindowRect(&rectWindow, dwStyle, FALSE);	// FALSE => No menu in window

	// Move back to our chosen position
	rectWindow = CRect(topLeftCorner.x, topLeftCorner.y, topLeftCorner.x + rectWindow.Width(),
							topLeftCorner.y + rectWindow.Height());
	// Flag to indicate whether to draw bounding box or not
	m_bDrawBB = bDrawBB;

	// Create default pen in blue and 2 pixels wide
	m_pen.CreatePen(PS_SOLID, 2, RGB(0,255,255));
	m_brush.CreateSolidBrush(RGB(255,0,0));

	// Now create the window
	return CFrameWnd::Create(NULL, strTitle, dwStyle, rectWindow);
}

void CImgWndBB::OnPaint() 
{
	// Proceed only if Dib is valid
	if (m_bValidDib == NEVER)
		return;

	CPaintDC dc(this); // device context for painting
	
	m_dib.Draw(&dc, 0, 0);

	if (m_bDrawBB)	{
		CPen *oldPen = dc.SelectObject(&m_pen);
		// Draw the eigen lines (cross for Vasi)
		dc.MoveTo( m_arrEigPoint[0] );
		dc.LineTo( m_arrEigPoint[1] );
		dc.MoveTo( m_arrEigPoint[2] );
		dc.LineTo( m_arrEigPoint[3] );
		dc.FrameRect(m_rect, &m_brush);
		dc.SelectObject(oldPen);
	}

	if (m_bValidDib == ONCE)
		m_bValidDib = NEVER;
	
	// Do not call CFrameWnd::OnPaint() for painting messages
}

