/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// TrackerDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Lafter.h"
#include "TrackerDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTrackerDialog dialog


CTrackerDialog::CTrackerDialog(CWnd* pParent /*=NULL*/)
: CDialog(CTrackerDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTrackerDialog)
	m_bCamera = FALSE;
	m_bZoom = FALSE;
	m_bSaveParams = FALSE;
	//}}AFX_DATA_INIT
	
	m_nSliderHeightRatio = 0;
	m_nSliderTheoHeightRatio = 0;
	m_nSliderP = 0;
	m_nD = 0;
	m_nSliderHeightThreshold = 0;
	m_nSliderTheoHeightThreshold = 0;
	m_nPanThreshold = 0;
	m_nTiltThreshold = 0;
	m_nNumFrames = 0;
	
	m_fHeightRatio = 0.0;
	m_fTheoHeightRatio = 0.0;
	m_fPError=0;
	m_fDError=0;
	m_fP = 0.0;
	m_fHeightThreshold=0.0;
	m_fTheoHeightThreshold=0.0;
	
	m_fXErr = 0.0f;
	m_fYErr = 0.0f;
	m_fPanSpeed = 0.0f;
	m_fTiltSpeed = 0.0f;
	m_fZoomRatio = 0.0f;
	m_bPanic = FALSE;
	
	m_pDlg = NULL;
	::InitializeCriticalSection(&m_Sem);
}

CTrackerDialog::CTrackerDialog(CDialog* pDlg)
{
	m_bCamera = FALSE;
	m_bZoom = FALSE;
	m_bSaveParams = FALSE;
	
	m_nSliderHeightRatio = 0;
	m_nSliderTheoHeightRatio = 0;
	m_nSliderP = 0;
	m_nD = 0;
	m_nSliderHeightThreshold = 0;
	m_nSliderTheoHeightThreshold = 0;
	m_nPanThreshold = 0;
	m_nTiltThreshold = 0;
	m_nNumFrames = 0;
	
	m_fHeightRatio = 0.0;
	m_fTheoHeightRatio = 0.0;
	m_fPError=0.0;
	m_fDError=0;
	m_fP = 0.0;
	m_fHeightThreshold=0.0;
	m_fTheoHeightThreshold=0.0;
	
	m_fXErr = 0.0f;
	m_fYErr = 0.0f;
	m_fPanSpeed = 0.0f;
	m_fTiltSpeed = 0.0f;
	m_fZoomRatio = 0.0f;
	m_bPanic = FALSE;
	
	m_pDlg = pDlg;
	::InitializeCriticalSection(&m_Sem);
}

CTrackerDialog::~CTrackerDialog()
{
	::DeleteCriticalSection(&m_Sem);
};

void CTrackerDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTrackerDialog)
	DDX_Check(pDX, IDC_CAMERA, m_bCamera);
	DDX_Check(pDX, IDC_ZOOM, m_bZoom);
	DDX_Check(pDX, IDC_SAVEPARAMS, m_bSaveParams);
	//}}AFX_DATA_MAP
}

BOOL CTrackerDialog::Create()
{
	return CDialog::Create(CTrackerDialog::IDD);
}

ErrorNumber	CTrackerDialog::Init()
{	
	CRect	auxRect;
	float	  fAux1, fAux2, fAux3;
	// Initializes the tracker
	ErrorNumber err =  m_tracker.Init();
	if ( err == kErrNoErr ) {
		m_tracker.GetParameters( &auxRect, &m_fHeightRatio, &m_fP, &fAux1,
			&fAux2, &fAux3,
			&m_fHeightThreshold, &m_fTheoHeightRatio,
			&m_fTheoHeightThreshold );
		PostMessage(WM_UPDATE_DATA, 0, 0);
		m_nD = (int)fAux1;
		m_nPanThreshold = (int)fAux2;
		m_nTiltThreshold = (int)fAux3;
	}
	return err;
}

void	CTrackerDialog:: CleanUp ()
{
	m_tracker.Dispose();
}

void	CTrackerDialog:: SetParameters  (CRect*  bounds,
										 float   heightRatio,
										 float   p,
										 float   d,
										 float   xThreshold,
										 float   yThreshold,
										 float   ratioThreshold,
										 float   theoRatio,
										 float   theoRatioThreshold)
{
	// Set its internal variables
	m_fHeightRatio = heightRatio;
	m_fP = p;
	m_nD = (int)d;
	m_nPanThreshold = (int)xThreshold;
	m_nTiltThreshold = (int)yThreshold;
	m_fTheoHeightRatio = theoRatio;
	m_fTheoHeightThreshold = theoRatioThreshold;
	
	PostMessage(WM_UPDATE_DATA, 0, 0);
	
	::EnterCriticalSection( &m_Sem );
	m_tracker.SetParameters( bounds, heightRatio, p, d, xThreshold, yThreshold, ratioThreshold, 
		theoRatio, theoRatioThreshold );
	::LeaveCriticalSection( &m_Sem );
}


BEGIN_MESSAGE_MAP(CTrackerDialog, CDialog)
//{{AFX_MSG_MAP(CTrackerDialog)
ON_WM_CLOSE()
ON_BN_CLICKED(IDC_CAMERA, OnCamera)
ON_BN_CLICKED(IDC_ZOOM, OnZoom)
ON_BN_CLICKED(IDC_SAVEPARAMS, OnSaveparams)
ON_WM_HSCROLL()
ON_WM_TIMER()
//}}AFX_MSG_MAP
ON_MESSAGE(WM_UPDATE_DATA, OnUpdateData)
ON_MESSAGE(WM_UPDATE_ERRORS, OnUpdateErrors)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrackerDialog message handlers

long CTrackerDialog::OnUpdateData(UINT wParam, long lParam)
{
	// Update all the edit controls that display the internal state of the camera
	
	// Update all the edit controls that display the internal state of the camera
	// Initialize Sliders
	CString strText;
	CSliderCtrl* pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_HEIGHTRATIO);
	pSlide->SetPos((int)(m_fHeightRatio*kTrackerDlgSliderMax));
	strText.Format("%1.2f", m_fArrayHeightRatio[pSlide->GetPos()]);
	SetDlgItemText(IDC_HEIGHTRATIO, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_THEOHEIGHTRATIO);
	pSlide->SetPos((int)(m_fTheoHeightRatio*kTrackerDlgSliderMax));
	strText.Format("%1.2f", m_fArrayTheoHeightRatio[pSlide->GetPos()]);
	SetDlgItemText(IDC_THEOHEIGHTRATIO, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_P);
	pSlide->SetPos((int)(m_fP*kTrackerDlgSliderMax));
	strText.Format("%2.1f", m_fArrayP[pSlide->GetPos()]);
	SetDlgItemText(IDC_P, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_D);
	pSlide->SetPos(m_nD);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_D, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_HEIGHTTHRESHOLD);
	pSlide->SetPos((int)(m_fHeightThreshold*kTrackerDlgSliderMax));
	strText.Format("%1.2f", m_fArrayHeightThreshold[pSlide->GetPos()]);
	SetDlgItemText(IDC_HEIGHTTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_THEOHEIGHTTHRESHOLD);
	pSlide->SetPos((int)(m_fTheoHeightThreshold*kTrackerDlgSliderMax));
	strText.Format("%1.2f", m_fArrayTheoHeightThreshold[pSlide->GetPos()]);
	SetDlgItemText(IDC_THEOHEIGHTTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_PANTHRESHOLD);
	pSlide->SetPos(m_nPanThreshold*kTrackerDlgSliderMax);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_PANTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_TILTTHRESHOLD);
	pSlide->SetPos(m_nTiltThreshold*kTrackerDlgSliderMax);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_TILTTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_NUMFRAMES);
	pSlide->SetPos(m_nNumFrames);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_NUMFRAMES, strText);
	
	if(m_pDlg != NULL) {
		m_pDlg->PostMessage(WM_TKUPDATE, 1);
	}
	return 0L;
}

long	CTrackerDialog::OnUpdateErrors( UINT wParam, long lParam )
{
	CString s;	
	::EnterCriticalSection( &m_Sem );
	m_tracker.GetState( m_fXErr, m_fYErr, m_fPanSpeed, m_fTiltSpeed, m_fZoomRatio, m_bPanic );
	::LeaveCriticalSection( &m_Sem );
	s.Format("%2.3f", m_fXErr);
	SetDlgItemText(IDC_PANERROR, s);
	s.Format("%2.3f", m_fYErr);
	SetDlgItemText(IDC_TILTERROR, s);
	s.Format("%2.3f", m_fPanSpeed);
	SetDlgItemText(IDC_PANSPEED, s);
	s.Format("%2.3f", m_fTiltSpeed);
	SetDlgItemText(IDC_TILTSPEED, s);
	s.Format("%2.3f", m_fZoomRatio);
	SetDlgItemText(IDC_ZOOMRATIO, s);
	s.Format("%s", (m_bPanic ? "TRUE" : "FALSE"));
	SetDlgItemText(IDC_PANIC, s);
	if(m_pDlg != NULL) {
		m_pDlg->PostMessage(WM_TKUPDATE, 1);
	}
	return 0L;
}

void CTrackerDialog::OnClose() 
{
	Hide();
	// TODO: Add your message handler code here and/or call default
	m_pDlg->PostMessage(WM_TKGOODBYE, 1);	
}

void CTrackerDialog::OnCamera() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	if(m_pDlg != NULL) {
		m_pDlg->PostMessage(WM_TKUPDATE, 1);
	}
}

void CTrackerDialog::OnZoom() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	if(m_pDlg != NULL) {
		m_pDlg->PostMessage(WM_TKUPDATE, 1);
	}	
}

void CTrackerDialog::OnSaveparams() 
{
	// TODO: Add your control notification handler code here
	
	if(m_pDlg != NULL) {
		m_pDlg->PostMessage(WM_TKUPDATE, 1);
	}
	// Set the tracker parameters
	::EnterCriticalSection( &m_Sem );
	m_tracker.SetParameters(NULL, m_fHeightRatio, m_fP, (float) m_nD, 
		(float) m_nPanThreshold, (float) m_nTiltThreshold, m_fHeightThreshold,
		m_fTheoHeightRatio, m_fTheoHeightThreshold );	
	::LeaveCriticalSection( &m_Sem );
}

BOOL CTrackerDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	
	//Initialize sliders arrays
	int i;
	double fHeight=0.0;
	double fP=0.0;
	for (i=0;i<=kTrackerDlgSliderMax;i++)
	{
		m_fArrayHeightRatio[i] = (float) fHeight;
		m_fArrayTheoHeightRatio[i] = (float) fHeight;
		m_fArrayP[i]= (float) fP;
		m_fArrayHeightThreshold[i] = (float) fHeight;
		m_fArrayTheoHeightThreshold[i] = (float) fHeight;
		fHeight = fHeight + 0.01;
		fP = fP + 0.1;
	}
	
	// Initialize Sliders
	CString strText;
	CSliderCtrl* pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_HEIGHTRATIO);
	
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nSliderHeightRatio);
	strText.Format("%1.2f", m_fArrayHeightRatio[pSlide->GetPos()]);
	SetDlgItemText(IDC_HEIGHTRATIO, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_THEOHEIGHTRATIO);
	
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nSliderTheoHeightRatio);
	strText.Format("%1.2f", m_fArrayTheoHeightRatio[pSlide->GetPos()]);
	SetDlgItemText(IDC_THEOHEIGHTRATIO, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_P);
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nSliderP);
	strText.Format("%2.1f", m_fArrayP[pSlide->GetPos()]);
	SetDlgItemText(IDC_P, strText);
	
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_D);
	pSlide->SetRange(0,500);
	pSlide->SetPos(m_nD);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_D, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_HEIGHTTHRESHOLD);
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nSliderHeightThreshold);
	strText.Format("%1.2f", m_fArrayHeightThreshold[pSlide->GetPos()]);
	SetDlgItemText(IDC_HEIGHTTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_THEOHEIGHTTHRESHOLD);
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nSliderTheoHeightThreshold);
	strText.Format("%1.2f", m_fArrayTheoHeightThreshold[pSlide->GetPos()]);
	SetDlgItemText(IDC_THEOHEIGHTTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_PANTHRESHOLD);
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nPanThreshold);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_PANTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_TILTTHRESHOLD);
	pSlide->SetRange(0,kTrackerDlgSliderMax);
	pSlide->SetPos(m_nTiltThreshold);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_TILTTHRESHOLD, strText);
	
	pSlide = (CSliderCtrl*) GetDlgItem(IDC_TRACK_NUMFRAMES);
	pSlide->SetRange(0,500);
	pSlide->SetPos(m_nNumFrames);
	strText.Format("%d", pSlide->GetPos());
	SetDlgItemText(IDC_NUMFRAMES, strText);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CTrackerDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// TODO: Add your message handler code here and/or call default
	CSliderCtrl* pSlide = (CSliderCtrl*) pScrollBar;
	CString strText;
	
	switch (pScrollBar->GetDlgCtrlID()) {
	case IDC_TRACK_THEOHEIGHTRATIO:
		m_nSliderTheoHeightRatio = pSlide->GetPos();
		m_fTheoHeightRatio = m_fArrayTheoHeightRatio[m_nSliderTheoHeightRatio];
		strText.Format("%1.2f", m_fTheoHeightRatio);
		SetDlgItemText(IDC_THEOHEIGHTRATIO, strText);
		break;
	case IDC_TRACK_HEIGHTRATIO:
		m_nSliderHeightRatio = pSlide->GetPos();
		m_fHeightRatio = m_fArrayHeightRatio[m_nSliderHeightRatio];
		strText.Format("%1.2f", m_fHeightRatio);
		SetDlgItemText(IDC_HEIGHTRATIO, strText);
		break;
	case IDC_TRACK_P:
		m_nSliderP = pSlide->GetPos();
		m_fP = m_fArrayP[m_nSliderP];
		strText.Format("%2.1f", m_fP);
		SetDlgItemText(IDC_P, strText);
		break;
	case IDC_TRACK_D:
		m_nD = pSlide->GetPos();
		strText.Format("%d", pSlide->GetPos());
		SetDlgItemText(IDC_D, strText);
		break;
	case IDC_TRACK_THEOHEIGHTTHRESHOLD:
		m_nSliderTheoHeightThreshold = pSlide->GetPos();
		m_fTheoHeightThreshold = m_fArrayTheoHeightThreshold[m_nSliderTheoHeightThreshold];
		strText.Format("%1.2f", m_fTheoHeightThreshold);
		SetDlgItemText(IDC_THEOHEIGHTTHRESHOLD, strText);
		break;
	case IDC_TRACK_HEIGHTTHRESHOLD:
		m_nSliderHeightThreshold = pSlide->GetPos();
		m_fHeightThreshold = m_fArrayHeightThreshold[m_nSliderHeightThreshold];
		strText.Format("%1.2f", m_fHeightThreshold);
		SetDlgItemText(IDC_HEIGHTTHRESHOLD, strText);
		break;
	case IDC_TRACK_PANTHRESHOLD:
		m_nPanThreshold = pSlide->GetPos();
		strText.Format("%d", pSlide->GetPos());
		SetDlgItemText(IDC_PANTHRESHOLD, strText);
		break;
	case IDC_TRACK_TILTTHRESHOLD:
		m_nTiltThreshold = pSlide->GetPos();
		strText.Format("%d", pSlide->GetPos());
		SetDlgItemText(IDC_TILTTHRESHOLD, strText);
		break;
	case IDC_TRACK_NUMFRAMES:
		m_nNumFrames = pSlide->GetPos();
		strText.Format("%d", pSlide->GetPos());
		SetDlgItemText(IDC_NUMFRAMES, strText);
		break;
	}
	
	if (m_pDlg != NULL) {
		m_pDlg->PostMessage(WM_TKUPDATE, 1);
	}
}

void CTrackerDialog::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	switch (nIDEvent) {
	case 1:
		OnUpdateErrors( 0, 0 );
		break;
	}
	
	CDialog::OnTimer(nIDEvent);
}

void CTrackerDialog::PostNcDestroy() 
{
	// TODO: Add your specialized code here and/or call the base class
	CleanUp();
	CDialog::PostNcDestroy();
}

BOOL CTrackerDialog::DestroyWindow() 
{
	// TODO: Add your specialized code here and/or call the base class
	m_tracker.Dispose();
	return CDialog::DestroyWindow();
}
