/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// UVector2.h
// Nuria Oliver 1997, nuria@media.mit.edu

#ifndef _UVECTOR2_H_
#define _UVECTOR2_H_

template<class T>
class TMatrix3;

template< class T >
class TVector2
{
    friend class TMatrix3<T>;
private:
    T *m_pAllocArray;
public:
    TVector2(){ m_pAllocArray = new T[2]; }
    TVector2(T e1, T e2){ m_pAllocArray = new T[2];set(e1, e2); }
    TVector2(TVector2<T>& v){ m_pAllocArray = new T[2];set(v); }
    ~TVector2() { if (m_pAllocArray) delete m_pAllocArray;}
	
	T* getData() { return m_pAllocArray; }
	T& operator()( int i ){return m_pAllocArray[i];}
	void setElem( int i, T val ){ m_pAllocArray[i]=val;}
    
	void set(T* v);
	void set(T e1, T e2);
	void set(TVector2<T>& v){set(v.getData() );}
	void setValue(T val);
	void setZero() {setValue(T(0));}
	void setAddEq(T e1, T e2);
	void setAddEq( T* v1 );
	void setAddEq( TVector2<T>& v1 ){ setAddEq( v1.getData() ); }
	void setAdd(T* v1, T* v2 );
	void setAdd(TVector2<T>& v1, TVector2<T>& v2 ){ setAdd(v1.getData(), v2.getData() ); }
	void setSubtractEq(T e1, T e2);
	void setSubtractEq(T* v1 );
	void setSubtractEq(TVector2<T>& v1 ){setSubtractEq(v1(0), v1(1));}
	void setSubtract(T* v1, T* v2 );
	void setScaleEq( T val );
	void setDivideEq( T val );
	void setMultiply( TMatrix3<T>& m, TVector2<T>& v );
	void setMultiplyEq( TMatrix3<T>& m );
	void setStringValue( char* str );
	
	float getInnerProduct( TVector2<T>& v );
	float getSquare( ){ return(getInnerProduct(*this)); }
	void  getStringValue(char* s);
	
	int operator==(  TVector2<T>& ) const;
	TVector2& operator=( TVector2<T>& op) { set(op); return(*this); }
	void operator+=( TVector2<T>& op) {setAddEq(op); }
	void operator-=( TVector2<T>& op) {setSubtractEq(op);}
	void operator=(  T val ) { setValue(val); }
	void operator/=(  T val ) { setDivideEq( val ); }
	void operator*=(  T val ) { setScaleEq( val ); }
	
	void Write( CFile	&f );
};

template<class T>
inline void TVector2<T>:: set( T* v )
{
    m_pAllocArray[0]=v[0];
    m_pAllocArray[1]=v[1];
}


template<class T>
inline void TVector2<T>:: set( T e1, T e2 )
{
    m_pAllocArray[0]=e1;
    m_pAllocArray[1]=e2;
}

template<class T> 
inline void TVector2<T>:: setValue( T val )
{
    set( val, val );
}

template<class T>
inline void TVector2<T>:: setAddEq( T* v )
{
    m_pAllocArray[0]+=v[0];
    m_pAllocArray[1]+=v[1];
}

template<class T>
inline void TVector2<T>:: setAddEq( T e1, T e2 )
{
    m_pAllocArray[0]+=e1;
    m_pAllocArray[1]+=e2;
}

template<class T>
inline void TVector2<T>:: setAdd( T* v1, T* v2 )
{
    m_pAllocArray[0]=v1[0]+v2[0];
    m_pAllocArray[1]=v1[1]+v2[1];
}


template<class T>
inline void TVector2<T>:: setSubtractEq( T* v )
{
    m_pAllocArray[0]-=v[0];
    m_pAllocArray[1]-=v[1];
}

template<class T>
inline void TVector2<T>:: setSubtractEq( T e1, T e2 )
{
    m_pAllocArray[0]-=e1;
    m_pAllocArray[1]-=e2;
}

template<class T>
inline void TVector2<T>:: setSubtract( T* v1, T* v2 )
{
    m_pAllocArray[0]=v1[0]-v2[0];
    m_pAllocArray[1]=v1[1]-v2[1];
}

template<class T>
inline void TVector2<T>:: setMultiply( TMatrix3<T>& m, TVector2<T>& v )
{
    m_pAllocArray[0]=m(0)*v(0)+m(1)*v(1);
    m_pAllocArray[1]=m(1)*v(0)+m(2)*v(1);
}


template<class T>
inline void TVector2<T>::setMultiplyEq( TMatrix3<T>& m )
{
    TVector2<T> old( *this );
    m_pAllocArray[0]=m(0)*old(0)+m(1)*old(1);
    m_pAllocArray[1]=m(1)*old(0)+m(2)*old(1);    
}


template<class T>
inline void TVector2<T>:: setScaleEq( T val )
{
    m_pAllocArray[0]*=val;
    m_pAllocArray[1]*=val;
}

template<class T>
inline void TVector2<T>:: setDivideEq( T val )
{
    m_pAllocArray[0]/=val;
    m_pAllocArray[1]/=val;
}

template<class T>
float TVector2<T>::getInnerProduct( TVector2<T>& v )
{
    return( v(0)*m_pAllocArray[0]+v(1)*m_pAllocArray[1] );
}

template<class T>
int
TVector2<T>::
operator==(  TVector2<T>& m ) const
{
    for ( register int i=0; i<3; i++ )
		if ( m_pAllocArray[i]!=m(i) ) return 0;
		return 1;
}

template<class T>
void
TVector2<T>::Write( CFile	&f )
{
	CString		buf;
	buf.Format("%f %f ", m_pAllocArray[0], m_pAllocArray[1]);
	f.Write( buf, buf.GetLength() );
}

#endif /* _TVector2_H_ */
