/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// Filename	: VDibBase.h
// Created	: Sunday, June 1st, 1997
// Description: Base class from which the Dib and DibSection classes will be derived.
// 
//  Vasanth Philomin 1997, vasi@cs.umd.edu
///////////////////////////////////////////////////////////////////////////

#ifndef _VDIBBASE_H
#define _VDIBBASE_H

#include "UErrors.h"

// maximum width and height of the destination of a rotated copy
const int         kBMMaxCols      = 1024;
const int         kBMMaxRows      = kBMMaxCols;
const ErrorNumber kErrCantHandleEncoding    = 1040;
const ErrorNumber kErrWrongSize             = 1041;
#ifndef M_PI
#define M_PI	3.14159265359
#endif

struct BITMAPINFO_8
{
	BITMAPINFOHEADER bmiHeader;
	RGBQUAD			 bmiColors[256];
	operator BITMAPINFO* () { return (BITMAPINFO*)this;};
	operator const BITMAPINFO* () const { return (const BITMAPINFO*)this;};
	operator BITMAPINFOHEADER* (){ return &bmiHeader; };
	RGBQUAD* ColorTable () { return bmiColors;};
	const RGBQUAD* ColorTable () const {return bmiColors; };
};

class VDibBase	: public CObject	{
public:
	CSize GetDimensions();
	VDibBase()	: m_bmHeader(), m_pBits(0), m_bOwnMemory(TRUE)	{ }
	~VDibBase()	{	
		if ( m_bOwnMemory && m_pBits )
			delete m_pBits;
	}
	
	virtual inline LONG  	Height() const	{ return m_bmHeader.bmiHeader.biHeight; }
	virtual inline LONG  	Width() const	{ return m_bmHeader.bmiHeader.biWidth; }
	virtual BITMAPINFO_8 	BitMapInfo() 	{ return m_bmHeader; }
	virtual int 				   PixelSize() const	{ return m_bmHeader.bmiHeader.biBitCount; }
	virtual int					   LineOffset() const { return m_nLineOffset; }
	virtual CRect				GetValidRect() const { return m_rect; }
	virtual void	   SetValidRect(const CRect &r) { m_rect = r; }
	virtual ErrorNumber RotatedCopy (VDibBase* dest, CRect* destRect,
		ULONG cx, ULONG cy,
		int width, int height, float angle);
	
	void AttachMemory(LPBYTE bits, BOOL delete_storage = FALSE)	{
		if (m_bOwnMemory && m_pBits)
			delete m_pBits;
		if (!delete_storage)
			m_bOwnMemory = FALSE;
		m_pBits = bits;
	}
	virtual inline LPVOID GetBits() { return m_pBits;}
	//	void Draw (CDC* pDC, UINT x, UINT y);
protected:
	BOOL m_bOwnMemory;
	BITMAPINFO_8 m_bmHeader;
	LPBYTE m_pBits;
	int m_nLineOffset;
	CRect	m_rect;
	
    // preallocated memory for rotated copy
    static int        fCols[];
    static int        fRows[];
	
    inline
		BOOL           InView (int cx, int cy, int w, int h);
    inline
		BOOL           InView (int cx, int cy, int w, int h,
		float angle);
	
    // method for rotated copy (don't even know their meaning)
    // bham stands for Bresenhams
    inline
		void              bham_array_pos (int *ys, short int dx,
		short int dy, short int mid);
    inline
		void              bham_array_neg ( int *ys, short int dx,
		short int dy, short int mid);
	
};

#endif